@@ -13,17 +13,13 @@ import numpy as np
 import numpy.linalg as npl
 from numpy.linalg import slogdet
 
-from statsmodels.tools.numdiff import (approx_hess, approx_fprime)
-from statsmodels.tools.decorators import cache_readonly
+from statsmodels.tools.numdiff import approx_hess, approx_fprime
 from statsmodels.tsa.vector_ar.irf import IRAnalysis
-from statsmodels.tsa.vector_ar.var_model import VARProcess, \
-                                                        VARResults
+from statsmodels.tsa.vector_ar.var_model import VARProcess, VARResults
 
 import statsmodels.tsa.vector_ar.util as util
 import statsmodels.tsa.base.tsa_model as tsbase
-from statsmodels.compat.numpy import np_matrix_rank
 
-mat = np.array
 
 def svar_ckerr(svar_type, A, B):
     if A is None and (svar_type == 'A' or svar_type == 'AB'):
@@ -32,6 +28,7 @@ def svar_ckerr(svar_type, A, B):
 
         raise ValueError('SVAR of type B or AB but B array not given.')
 
+
 class SVAR(tsbase.TimeSeriesModel):
     r"""
     Fit VAR and then estimate structural components of A and B, defined:
@@ -74,8 +71,11 @@ class SVAR(tsbase.TimeSeriesModel):
 
         svar_ckerr(svar_type, A, B)
 
-        #initialize A, B as I if not given
-        #Initialize SVAR masks
+        self.A_original = A
+        self.B_original = B
+
+        # initialize A, B as I if not given
+        # Initialize SVAR masks
         if A is None:
             A = np.identity(self.neqs)
             self.A_mask = A_mask = np.zeros(A.shape, dtype=bool)
@@ -166,8 +166,8 @@ class SVAR(tsbase.TimeSeriesModel):
         if ic is not None:
             selections = self.select_order(maxlags=maxlags, verbose=verbose)
             if ic not in selections:
-                raise Exception("%s not recognized, must be among %s"
-                                % (ic, sorted(selections)))
+                raise ValueError("%s not recognized, must be among %s"
+                                 % (ic, sorted(selections)))
             lags = selections[ic]
             if verbose:
                 print('Using %d based on %s criterion' %  (lags, ic))
@@ -184,7 +184,6 @@ class SVAR(tsbase.TimeSeriesModel):
                                    solver=solver, override=override,
                                    maxiter=maxiter, maxfun=maxfun)
 
-
     def _get_init_params(self, A_guess, B_guess):
         """
         Returns either the given starting or .1 if none are given.
@@ -250,15 +249,15 @@ class SVAR(tsbase.TimeSeriesModel):
         self.sigma_u = omega
 
         A, B = self._solve_AB(start_params, override=override,
-                                                    solver=solver,
-                                                    maxiter=maxiter,
-                                                    maxfun=maxfun)
+                              solver=solver,
+                              maxiter=maxiter,
+                              maxfun=maxfun)
         A_mask = self.A_mask
         B_mask = self.B_mask
 
         return SVARResults(y, z, var_params, omega, lags,
-                            names=self.endog_names, trend=trend,
-                            dates=self.data.dates, model=self,
+                           names=self.endog_names, trend=trend,
+                           dates=self.data.dates, model=self,
                            A=A, B=B, A_mask=A_mask, B_mask=B_mask)
 
     def loglike(self, params):
@@ -294,10 +293,9 @@ class SVAR(tsbase.TimeSeriesModel):
         sign, b_logdet = slogdet(B**2) #numpy 1.4 compat
         b_slogdet = sign * b_logdet
 
-        likl = -nobs/2. * (neqs * np.log(2 * np.pi) - \
-                np.log(npl.det(A)**2) + b_slogdet + \
-                np.trace(trc_in))
-
+        likl = -nobs/2. * (neqs * np.log(2 * np.pi) -
+                           np.log(npl.det(A)**2) + b_slogdet +
+                           np.trace(trc_in))
 
         return likl
 
@@ -316,7 +314,6 @@ class SVAR(tsbase.TimeSeriesModel):
         loglike = self.loglike
         return approx_fprime(AB_mask, loglike, epsilon=1e-8)
 
-
     def hessian(self, AB_mask):
         """
         Returns numerical hessian.
@@ -325,7 +322,7 @@ class SVAR(tsbase.TimeSeriesModel):
         return approx_hess(AB_mask, loglike)
 
     def _solve_AB(self, start_params, maxiter, maxfun, override=False,
-            solver='bfgs'):
+                  solver='bfgs'):
         """
         Solves for MLE estimate of structural parameters
 
@@ -359,7 +356,7 @@ class SVAR(tsbase.TimeSeriesModel):
         A[A_mask] = start_params[:A_len]
         B[B_mask] = start_params[A_len:]
 
-        if override == False:
+        if not override:
             J = self._compute_J(A, B)
             self.check_order(J)
             self.check_rank(J)
@@ -367,10 +364,9 @@ class SVAR(tsbase.TimeSeriesModel):
             print("Order/rank conditions have not been checked")
 
         retvals = super(SVAR, self).fit(start_params=start_params,
-                    method=solver, maxiter=maxiter,
-                    maxfun=maxfun, ftol=1e-20, disp=0).params
-
-
+                                        method=solver, maxiter=maxiter,
+                                        maxfun=maxfun, ftol=1e-20,
+                                        disp=0).params
 
         A[A_mask] = retvals[:A_len]
         B[B_mask] = retvals[A_len:]
@@ -414,16 +410,16 @@ class SVAR(tsbase.TimeSeriesModel):
 
         j = 0
         j_d = 0
-        if len(A_solve[A_mask]) is not 0:
+        if len(A_solve[A_mask]) != 0:
             A_vec = np.ravel(A_mask, order='F')
             for k in range(neqs**2):
-                if A_vec[k] == True:
+                if A_vec[k]:
                     S_B[k,j] = -1
                     j += 1
-        if len(B_solve[B_mask]) is not 0:
+        if len(B_solve[B_mask]) != 0:
             B_vec = np.ravel(B_mask, order='F')
             for k in range(neqs**2):
-                if B_vec[k] == True:
+                if B_vec[k]:
                     S_D[k,j_d] = 1
                     j_d +=1
 
@@ -443,11 +439,12 @@ class SVAR(tsbase.TimeSeriesModel):
                              "solution may not be unique")
 
     def check_rank(self, J):
-        rank = np_matrix_rank(J)
+        rank = np.linalg.matrix_rank(J)
         if rank < np.size(J, axis=1):
             raise ValueError("Rank condition not met: "
                              "solution may not be unique.")
 
+
 class SVARProcess(VARProcess):
     """
     Class represents a known SVAR(p) process
@@ -462,10 +459,6 @@ class SVARProcess(VARProcess):
     A_mask : neqs x neqs mask array with known parameters masked
     B : neqs x neqs np.ndarry with unknown parameters marked with 'E'
     B_mask : neqs x neqs mask array with known parameters masked
-
-    Returns
-    -------
-    **Attributes**:
     """
     def __init__(self, coefs, intercept, sigma_u, A_solve, B_solve,
                  names=None):
@@ -479,7 +472,6 @@ class SVARProcess(VARProcess):
         self.names = names
 
     def orth_ma_rep(self, maxn=10, P=None):
-
         """
 
         Unavailable for SVAR
@@ -500,7 +492,8 @@ class SVARProcess(VARProcess):
             P = np.dot(npl.inv(A_solve), B_solve)
 
         ma_mats = self.ma_rep(maxn=maxn)
-        return mat([np.dot(coefs, P) for coefs in ma_mats])
+        return np.array([np.dot(coefs, P) for coefs in ma_mats])
+
 
 class SVARResults(SVARProcess, VARResults):
     """
@@ -519,10 +512,8 @@ class SVARResults(SVARProcess, VARResults):
         List of names of the endogenous variables in order of appearance in `endog`.
     dates
 
-
-    Returns
-    -------
-    **Attributes**
+    Attributes
+    ----------
     aic
     bic
     bse
@@ -609,8 +600,8 @@ class SVARResults(SVARProcess, VARResults):
         self.A_mask = A_mask
         self.B_mask = B_mask
 
-        super(SVARResults, self).__init__(coefs, intercept, sigma_u, A,
-                             B, names=names)
+        super(SVARResults, self).__init__(coefs, intercept, sigma_u, A, B,
+                                          names=names)
 
     def irf(self, periods=10, var_order=None):
         """
@@ -676,68 +667,44 @@ class SVARResults(SVARProcess, VARResults):
         B = self.B
         A_mask = self.A_mask
         B_mask = self.B_mask
-        A_pass = np.zeros(A.shape, dtype='|S1')
-        B_pass = np.zeros(B.shape, dtype='|S1')
-        A_pass[~A_mask] = A[~A_mask]
-        B_pass[~B_mask] = B[~B_mask]
-        A_pass[A_mask] = 'E'
-        B_pass[B_mask] = 'E'
-        if A_mask.sum() == 0:
-            s_type = 'B'
-        elif B_mask.sum() == 0:
-            s_type = 'A'
-        else:
-            s_type = 'AB'
+        A_pass = self.model.A_original
+        B_pass = self.model.B_original
+        s_type = self.model.svar_type
+
         g_list = []
 
+        def agg(impulses):
+            if cum:
+                return impulses.cumsum(axis=0)
+            return impulses
 
+        opt_A = A[A_mask]
+        opt_B = B[B_mask]
         for i in range(repl):
-            #discard first hundred to correct for starting bias
-            sim = util.varsim(coefs, intercept, sigma_u,
-                    steps=nobs+burn)
+            # discard first hundred to correct for starting bias
+            sim = util.varsim(coefs, intercept, sigma_u, seed=seed,
+                              steps=nobs + burn)
             sim = sim[burn:]
-            if cum == True:
-                if i < 10:
-                    sol = SVAR(sim, svar_type=s_type, A=A_pass,
-                               B=B_pass).fit(maxlags=k_ar)
-                    g_list.append(np.append(sol.A[sol.A_mask].\
-                                            tolist(),
-                                            sol.B[sol.B_mask].\
-                                            tolist()))
-                    ma_coll[i] = sol.svar_ma_rep(maxn=T).cumsum(axis=0)
-                elif i >= 10:
-                    if i == 10:
-                        mean_AB = np.mean(g_list, axis = 0)
-                        split = len(A_pass[A_mask])
-                        opt_A = mean_AB[:split]
-                        opt_A = mean_AB[split:]
-                    ma_coll[i] = SVAR(sim, svar_type=s_type, A=A_pass,
-                                 B=B_pass).fit(maxlags=k_ar,\
-                                 A_guess=opt_A, B_guess=opt_B).\
-                                 svar_ma_rep(maxn=T).cumsum(axis=0)
 
+            smod = SVAR(sim, svar_type=s_type, A=A_pass, B=B_pass)
+            if i == 10:
+                # Use first 10 to update starting val for remainder of fits
+                mean_AB = np.mean(g_list, axis=0)
+                split = len(A[A_mask])
+                opt_A = mean_AB[:split]
+                opt_B = mean_AB[split:]
 
-            elif cum == False:
-                if i < 10:
-                    sol = SVAR(sim, svar_type=s_type, A=A_pass,
-                               B=B_pass).fit(maxlags=k_ar)
-                    g_list.append(np.append(sol.A[A_mask].tolist(),
-                                            sol.B[B_mask].tolist()))
-                    ma_coll[i] = sol.svar_ma_rep(maxn=T)
-                elif i >= 10:
-                    if i == 10:
-                        mean_AB = np.mean(g_list, axis = 0)
-                        split = len(A[A_mask])
-                        opt_A = mean_AB[:split]
-                        opt_B = mean_AB[split:]
-                    ma_coll[i] = SVAR(sim, svar_type=s_type, A=A_pass,
-                                 B=B_pass).fit(maxlags=k_ar,\
-                                 A_guess = opt_A, B_guess = opt_B).\
-                                 svar_ma_rep(maxn=T)
+            sres = smod.fit(maxlags=k_ar, A_guess=opt_A, B_guess=opt_B)
 
-        ma_sort = np.sort(ma_coll, axis=0) #sort to get quantiles
-        index = round(signif/2*repl)-1,round((1-signif/2)*repl)-1
-        lower = ma_sort[index[0],:, :, :]
-        upper = ma_sort[index[1],:, :, :]
+            if i < 10:
+                # save estimates for starting val if in first 10
+                g_list.append(np.append(sres.A[A_mask].tolist(),
+                                        sres.B[B_mask].tolist()))
+            ma_coll[i] = agg(sres.svar_ma_rep(maxn=T))
+
+        ma_sort = np.sort(ma_coll, axis=0)  # sort to get quantiles
+        index = (int(round(signif / 2 * repl) - 1),
+                 int(round((1 - signif / 2) * repl) - 1))
+        lower = ma_sort[index[0], :, :, :]
+        upper = ma_sort[index[1], :, :, :]
         return lower, upper
-
